package kr.co.vividnext.sodalive.admin.audition.role

import com.querydsl.jpa.impl.JPAQueryFactory
import kr.co.vividnext.sodalive.admin.audition.GetAuditionRoleListData
import kr.co.vividnext.sodalive.admin.audition.QGetAuditionRoleListData
import kr.co.vividnext.sodalive.audition.AuditionRole
import kr.co.vividnext.sodalive.audition.QAudition.audition
import kr.co.vividnext.sodalive.audition.QAuditionApplicant.auditionApplicant
import kr.co.vividnext.sodalive.audition.QAuditionRole.auditionRole
import kr.co.vividnext.sodalive.audition.QAuditionVote.auditionVote
import kr.co.vividnext.sodalive.member.QMember.member
import org.springframework.beans.factory.annotation.Value
import org.springframework.data.jpa.repository.JpaRepository

interface AdminAuditionRoleRepository : JpaRepository<AuditionRole, Long>, AdminAuditionRoleQueryRepository

interface AdminAuditionRoleQueryRepository {
    fun getAuditionRoleListByAuditionId(auditionId: Long): List<GetAuditionRoleListData>
    fun getAuditionRoleDetail(auditionRoleId: Long): GetAuditionRoleDetailResponse
    fun getAuditionApplicantList(auditionRoleId: Long, offset: Long, limit: Long): List<GetAuditionRoleApplicantItem>
    fun getAuditionApplicantTotalCount(auditionRoleId: Long): Int
}

class AdminAuditionRoleQueryRepositoryImpl(
    private val queryFactory: JPAQueryFactory,

    @Value("\${cloud.aws.cloud-front.host}")
    private val cloudfrontHost: String
) : AdminAuditionRoleQueryRepository {
    override fun getAuditionRoleListByAuditionId(auditionId: Long): List<GetAuditionRoleListData> {
        return queryFactory
            .select(
                QGetAuditionRoleListData(
                    auditionRole.id,
                    auditionRole.name,
                    auditionRole.imagePath.prepend("/").prepend(cloudfrontHost),
                    auditionRole.information,
                    auditionRole.auditionScriptUrl,
                    auditionRole.status
                )
            )
            .from(auditionRole)
            .innerJoin(auditionRole.audition, audition)
            .where(
                auditionRole.audition.id.eq(auditionId),
                auditionRole.isActive.isTrue
            )
            .fetch()
    }

    override fun getAuditionRoleDetail(auditionRoleId: Long): GetAuditionRoleDetailResponse {
        return queryFactory
            .select(
                QGetAuditionRoleDetailResponse(
                    auditionRole.name,
                    auditionRole.imagePath.prepend("/").prepend(cloudfrontHost),
                    auditionRole.information,
                    auditionRole.auditionScriptUrl
                )
            )
            .from(auditionRole)
            .where(auditionRole.id.eq(auditionRoleId))
            .fetchFirst()
    }

    override fun getAuditionApplicantList(
        auditionRoleId: Long,
        offset: Long,
        limit: Long
    ): List<GetAuditionRoleApplicantItem> {
        return queryFactory
            .select(
                QGetAuditionRoleApplicantItem(
                    auditionApplicant.id,
                    member.nickname,
                    member.profileImage.prepend("/").prepend(cloudfrontHost),
                    auditionApplicant.phoneNumber,
                    auditionApplicant.voicePath.prepend("/").prepend(cloudfrontHost),
                    auditionVote.id.count()
                )
            )
            .from(auditionApplicant)
            .innerJoin(auditionApplicant.member, member)
            .innerJoin(auditionApplicant.role, auditionRole)
            .leftJoin(auditionVote).on(auditionApplicant.id.eq(auditionVote.applicant.id))
            .where(
                auditionRole.id.eq(auditionRoleId),
                auditionApplicant.isActive.isTrue
            )
            .groupBy(auditionApplicant.id)
            .orderBy(auditionVote.id.count().desc())
            .offset(offset)
            .limit(limit)
            .fetch()
    }

    override fun getAuditionApplicantTotalCount(auditionRoleId: Long): Int {
        return queryFactory
            .select(auditionApplicant.id)
            .from(auditionApplicant)
            .innerJoin(auditionApplicant.role, auditionRole)
            .where(auditionRole.id.eq(auditionRoleId))
            .fetch()
            .size
    }
}
