package kr.co.vividnext.sodalive.admin.calculate.ratio

import com.querydsl.jpa.impl.JPAQueryFactory
import kr.co.vividnext.sodalive.admin.calculate.ratio.QCreatorSettlementRatio.creatorSettlementRatio
import kr.co.vividnext.sodalive.member.QMember.member
import org.springframework.data.jpa.repository.JpaRepository

interface CreatorSettlementRatioRepository :
    JpaRepository<CreatorSettlementRatio, Long>,
    CreatorSettlementRatioQueryRepository

interface CreatorSettlementRatioQueryRepository {
    fun getCreatorSettlementRatio(offset: Long, limit: Long): List<GetCreatorSettlementRatioItem>
    fun getCreatorSettlementRatioTotalCount(): Int
}

class CreatorSettlementRatioQueryRepositoryImpl(
    private val queryFactory: JPAQueryFactory
) : CreatorSettlementRatioQueryRepository {
    override fun getCreatorSettlementRatio(offset: Long, limit: Long): List<GetCreatorSettlementRatioItem> {
        return queryFactory
            .select(
                QGetCreatorSettlementRatioItem(
                    member.nickname,
                    creatorSettlementRatio.subsidy,
                    creatorSettlementRatio.liveSettlementRatio,
                    creatorSettlementRatio.contentSettlementRatio,
                    creatorSettlementRatio.communitySettlementRatio
                )
            )
            .from(creatorSettlementRatio)
            .innerJoin(creatorSettlementRatio.member, member)
            .orderBy(creatorSettlementRatio.id.asc())
            .offset(offset)
            .limit(limit)
            .fetch()
    }

    override fun getCreatorSettlementRatioTotalCount(): Int {
        return queryFactory
            .select(creatorSettlementRatio.id)
            .from(creatorSettlementRatio)
            .fetch()
            .size
    }
}
