package kr.co.vividnext.sodalive.audition

import com.querydsl.jpa.impl.JPAQueryFactory
import kr.co.vividnext.sodalive.audition.QAudition.audition
import org.springframework.beans.factory.annotation.Value
import org.springframework.data.jpa.repository.JpaRepository

interface AuditionRepository : JpaRepository<Audition, Long>, AuditionQueryRepository

interface AuditionQueryRepository {
    fun getInProgressAuditionCount(isAdult: Boolean): Int
    fun getCompletedAuditionCount(isAdult: Boolean): Int
    fun getAuditionList(offset: Long, limit: Long, isAdult: Boolean): List<GetAuditionListItem>
    fun getAuditionDetail(auditionId: Long): GetAuditionDetailRawData
}

class AuditionQueryRepositoryImpl(
    private val queryFactory: JPAQueryFactory,

    @Value("\${cloud.aws.cloud-front.host}")
    private val cloudFrontHost: String
) : AuditionQueryRepository {
    override fun getInProgressAuditionCount(isAdult: Boolean): Int {
        var where = audition.isActive.isTrue
            .and(audition.status.eq(AuditionStatus.IN_PROGRESS))

        if (!isAdult) {
            where = where.and(audition.isAdult.isFalse)
        }

        return queryFactory
            .select(audition.id)
            .from(audition)
            .where(where)
            .fetch()
            .size
    }

    override fun getCompletedAuditionCount(isAdult: Boolean): Int {
        var where = audition.isActive.isTrue
            .and(audition.status.eq(AuditionStatus.COMPLETED))

        if (!isAdult) {
            where = where.and(audition.isAdult.isFalse)
        }

        return queryFactory
            .select(audition.id)
            .from(audition)
            .where(where)
            .fetch()
            .size
    }

    override fun getAuditionList(offset: Long, limit: Long, isAdult: Boolean): List<GetAuditionListItem> {
        var where = audition.isActive.isTrue
            .and(
                audition.status.eq(AuditionStatus.COMPLETED)
                    .or(audition.status.eq(AuditionStatus.IN_PROGRESS))
            )

        if (!isAdult) {
            where = where.and(audition.isAdult.isFalse)
        }

        return queryFactory
            .select(
                QGetAuditionListItem(
                    audition.id,
                    audition.title,
                    audition.imagePath.prepend("/").prepend(cloudFrontHost),
                    audition.status.eq(AuditionStatus.COMPLETED)
                )
            )
            .from(audition)
            .where(where)
            .offset(offset)
            .limit(limit)
            .orderBy(audition.status.desc())
            .fetch()
    }

    override fun getAuditionDetail(auditionId: Long): GetAuditionDetailRawData {
        return queryFactory
            .select(
                QGetAuditionDetailRawData(
                    audition.id,
                    audition.title,
                    audition.imagePath.prepend("/").prepend(cloudFrontHost),
                    audition.information
                )
            )
            .from(audition)
            .where(audition.id.eq(auditionId))
            .fetchFirst()
    }
}
