package kr.co.vividnext.sodalive.audition.vote

import kr.co.vividnext.sodalive.audition.AuditionVote
import kr.co.vividnext.sodalive.audition.applicant.AuditionApplicantRepository
import kr.co.vividnext.sodalive.can.payment.CanPaymentService
import kr.co.vividnext.sodalive.can.use.CanUsage
import kr.co.vividnext.sodalive.common.SodaException
import kr.co.vividnext.sodalive.member.Member
import org.springframework.data.repository.findByIdOrNull
import org.springframework.stereotype.Service
import java.time.ZoneId
import java.time.ZoneOffset
import java.time.ZonedDateTime

@Service
class AuditionVoteService(
    private val repository: AuditionVoteRepository,
    private val applicantRepository: AuditionApplicantRepository,
    private val canPaymentService: CanPaymentService
) {
    fun voteAuditionApplicant(applicantId: Long, timezone: String, container: String, member: Member) {
        val applicant = applicantRepository.findByIdOrNull(applicantId)
            ?: throw SodaException("잘못된 요청입니다.\n다시 시도해 주세요.")

        val defaultZoneId = ZoneId.of("Asia/Seoul")
        val clientZoneId = try {
            ZoneId.of(timezone)
        } catch (e: Exception) {
            defaultZoneId
        }

        val nowInClientZone = ZonedDateTime.now(clientZoneId)
        val startOfDayClient = nowInClientZone.toLocalDate().atStartOfDay(clientZoneId)
        val endOfDayClient = startOfDayClient.plusDays(1).minusSeconds(1)

        val startDate = startOfDayClient.withZoneSameInstant(ZoneOffset.UTC).toLocalDateTime()
        val endDate = endOfDayClient.withZoneSameInstant(ZoneOffset.UTC).toLocalDateTime()

        val voteCount = repository.countByMemberIdAndVoteDateRange(
            memberId = member.id!!,
            startDate = startDate,
            endDate = endDate
        )

        if (voteCount > 100) {
            throw SodaException("오늘 응원은 여기까지!\n하루 최대 100회까지 응원이 가능합니다.\n내일 다시 이용해주세요.")
        }

        if (voteCount > 0) {
            canPaymentService.spendCan(
                memberId = member.id!!,
                needCan = 1,
                canUsage = CanUsage.AUDITION_VOTE,
                auditionApplicant = applicant,
                container = container
            )
        }

        val auditionVote = AuditionVote()
        auditionVote.applicant = applicant
        auditionVote.member = member
        repository.save(auditionVote)
    }
}
