package kr.co.vividnext.sodalive.content.category

import com.querydsl.jpa.impl.JPAQueryFactory
import kr.co.vividnext.sodalive.content.category.QCategory.category
import org.springframework.data.jpa.repository.JpaRepository

interface CategoryRepository : JpaRepository<Category, Long>, CategoryQueryRepository

interface CategoryQueryRepository {
    fun findByTitleAndMemberId(title: String, memberId: Long): Category?

    fun findByIdAndMemberId(categoryId: Long, memberId: Long): Category?

    fun findByCreatorId(creatorId: Long): List<GetCategoryListResponse>
}

class CategoryQueryRepositoryImpl(private val queryFactory: JPAQueryFactory) : CategoryQueryRepository {
    override fun findByTitleAndMemberId(title: String, memberId: Long): Category? {
        return queryFactory
            .selectFrom(category)
            .where(
                category.title.eq(title)
                    .and(category.member.id.eq(memberId))
            )
            .fetchFirst()
    }

    override fun findByIdAndMemberId(categoryId: Long, memberId: Long): Category? {
        return queryFactory
            .selectFrom(category)
            .where(
                category.id.eq(categoryId)
                    .and(category.member.id.eq(memberId))
            )
            .fetchFirst()
    }

    override fun findByCreatorId(creatorId: Long): List<GetCategoryListResponse> {
        return queryFactory
            .select(QGetCategoryListResponse(category.id, category.title))
            .from(category)
            .where(
                category.member.id.eq(creatorId)
                    .and(category.isActive.isTrue)
            )
            .orderBy(category.orders.asc())
            .fetch()
    }
}
