package kr.co.vividnext.sodalive.content.main.tab.free

import kr.co.vividnext.sodalive.content.AudioContentRepository
import kr.co.vividnext.sodalive.content.ContentType
import kr.co.vividnext.sodalive.content.main.GetAudioContentMainItem
import kr.co.vividnext.sodalive.content.main.GetAudioContentRankingItem
import kr.co.vividnext.sodalive.content.main.banner.AudioContentBannerService
import kr.co.vividnext.sodalive.content.main.curation.AudioContentCurationQueryRepository
import kr.co.vividnext.sodalive.content.main.tab.AudioContentMainTabRepository
import kr.co.vividnext.sodalive.content.main.tab.GetContentCurationResponse
import kr.co.vividnext.sodalive.content.main.tab.RecommendSeriesRepository
import kr.co.vividnext.sodalive.content.theme.AudioContentThemeQueryRepository
import kr.co.vividnext.sodalive.member.Member
import kr.co.vividnext.sodalive.rank.RankingService
import org.springframework.stereotype.Service

@Service
class AudioContentMainTabFreeService(
    private val repository: AudioContentMainTabRepository,
    private val bannerService: AudioContentBannerService,
    private val recommendSeriesRepository: RecommendSeriesRepository,
    private val curationRepository: AudioContentCurationQueryRepository,
    private val contentRepository: AudioContentRepository,
    private val rankingService: RankingService,
    private val audioContentRepository: AudioContentRepository,
    private val audioContentThemeRepository: AudioContentThemeQueryRepository
) {
    fun fetchData(
        isAdultContentVisible: Boolean,
        contentType: ContentType,
        member: Member
    ): GetContentMainTabFreeResponse {
        val isAdult = member.auth != null && isAdultContentVisible
        val memberId = member.id!!
        val tabId = 7L

        val contentBannerList = bannerService.getBannerList(
            tabId = tabId,
            memberId = memberId,
            isAdult = isAdult
        )

        val introduceCreator = curationRepository.findByContentMainTabIdAndTitle(
            tabId = tabId,
            title = "크리에이터 소개",
            isAdult = isAdult
        ).map {
            GetContentCurationResponse(
                title = it.title,
                items = contentRepository.findAudioContentByCurationIdV2(
                    curationId = it.id!!,
                    memberId = memberId,
                    isAdult = isAdult,
                    contentType = contentType
                )
            )
        }.filter { it.items.isNotEmpty() }

        val recommendSeriesList = recommendSeriesRepository.getRecommendSeriesList(
            isAdult = isAdult,
            contentType = contentType
        )

        val themeList = audioContentThemeRepository.getActiveThemeOfContent(
            isAdult = isAdult,
            isFree = true,
            contentType = contentType
        ).filter {
            it != "모닝콜" &&
                it != "알람" &&
                it != "슬립콜" &&
                it != "다시듣기" &&
                it != "ASMR" &&
                it != "릴레이" &&
                it != "챌린지" &&
                it != "자기소개"
        }
        val newFreeContentList = if (themeList.isNotEmpty()) {
            audioContentRepository.findByTheme(
                memberId = member.id!!,
                theme = themeList,
                isAdult = isAdult,
                contentType = contentType,
                offset = 0,
                limit = 10,
                isFree = true
            )
        } else {
            emptyList()
        }

        val creatorList = repository.findCreatorWithHasFreeContent(memberId, 4, isAdult, contentType)
        val playCountRankContentList = if (creatorList.isNotEmpty()) {
            rankingService.fetchFreeContentByCreatorIdTop4(creatorList[0].creatorId, isAdult, contentType)
        } else {
            emptyList()
        }

        val curationList = curationRepository.findByContentMainTabId(tabId = tabId, isAdult = isAdult)
            .filter { !it.title.contains("크리에이터 소개") }
            .map {
                GetContentCurationResponse(
                    title = it.title,
                    items = contentRepository.findAudioContentByCurationIdV2(
                        curationId = it.id!!,
                        memberId = memberId,
                        isAdult = isAdult,
                        contentType = contentType
                    )
                )
            }
            .filter { it.items.isNotEmpty() }

        return GetContentMainTabFreeResponse(
            contentBannerList = contentBannerList,
            introduceCreator = if (introduceCreator.isNotEmpty()) {
                introduceCreator[0]
            } else {
                null
            },
            recommendSeriesList = recommendSeriesList,
            themeList = themeList,
            newFreeContentList = newFreeContentList,
            creatorList = creatorList,
            playCountRankContentList = playCountRankContentList,
            curationList = curationList
        )
    }

    fun getIntroduceCreator(
        member: Member,
        isAdultContentVisible: Boolean,
        contentType: ContentType,
        offset: Long,
        limit: Long
    ): List<GetAudioContentMainItem> {
        val isAdult = member.auth != null && isAdultContentVisible
        val memberId = member.id!!

        val introduceCreatorCuration = curationRepository.findByContentMainTabIdAndTitle(
            tabId = 7L,
            title = "크리에이터 소개",
            isAdult = isAdult
        )

        return if (introduceCreatorCuration.isNotEmpty()) {
            contentRepository.findAudioContentByCurationIdV2(
                curationId = introduceCreatorCuration[0].id!!,
                memberId = memberId,
                isAdult = isAdult,
                contentType = contentType,
                offset = offset,
                limit = limit
            )
        } else {
            emptyList()
        }
    }

    fun getNewContentByTheme(
        theme: String,
        isAdultContentVisible: Boolean,
        contentType: ContentType,
        member: Member,
        offset: Long,
        limit: Long
    ): List<GetAudioContentMainItem> {
        return audioContentRepository.findByTheme(
            memberId = member.id!!,
            theme = if (theme.isNotBlank()) {
                listOf(theme)
            } else {
                audioContentThemeRepository.getActiveThemeOfContent(
                    isAdult = member.auth != null && isAdultContentVisible,
                    isFree = true,
                    contentType = contentType
                ).filter {
                    it != "모닝콜" &&
                        it != "알람" &&
                        it != "슬립콜" &&
                        it != "다시듣기" &&
                        it != "ASMR" &&
                        it != "릴레이" &&
                        it != "챌린지" &&
                        it != "자기소개"
                }
            },
            isAdult = member.auth != null && isAdultContentVisible,
            contentType = contentType,
            offset = offset,
            limit = limit,
            isFree = true
        )
    }

    fun getPopularContentByCreator(
        creatorId: Long,
        isAdult: Boolean,
        contentType: ContentType
    ): List<GetAudioContentRankingItem> {
        return rankingService.fetchFreeContentByCreatorIdTop4(creatorId, isAdult, contentType)
    }
}
