package kr.co.vividnext.sodalive.content.order

import com.querydsl.jpa.impl.JPAQueryFactory
import kr.co.vividnext.sodalive.content.QAudioContent.audioContent
import kr.co.vividnext.sodalive.content.order.QLimitedEditionOrder.limitedEditionOrder
import kr.co.vividnext.sodalive.content.order.QOrder.order
import kr.co.vividnext.sodalive.member.QMember.member
import org.springframework.data.jpa.repository.JpaRepository

interface LimitedEditionOrderRepository : JpaRepository<LimitedEditionOrder, Long>, LimitedEditionOrderQueryRepository

interface LimitedEditionOrderQueryRepository {
    fun getNextSequence(contentId: Long): Int

    fun getOrderSequence(contentId: Long, memberId: Long): Int?
}

class LimitedEditionOrderQueryRepositoryImpl(
    private val queryFactory: JPAQueryFactory
) : LimitedEditionOrderQueryRepository {
    override fun getNextSequence(contentId: Long): Int {
        val maxSequence = queryFactory.select(limitedEditionOrder.sequence)
            .from(limitedEditionOrder)
            .innerJoin(limitedEditionOrder.audioContent, audioContent)
            .where(limitedEditionOrder.audioContent.id.eq(contentId))
            .orderBy(limitedEditionOrder.sequence.desc())
            .limit(1)
            .fetchFirst()

        return if (maxSequence == null) {
            1
        } else {
            maxSequence + 1
        }
    }

    override fun getOrderSequence(contentId: Long, memberId: Long): Int? {
        return queryFactory.select(limitedEditionOrder.sequence)
            .from(limitedEditionOrder)
            .innerJoin(limitedEditionOrder.audioContent, audioContent)
            .innerJoin(limitedEditionOrder.order, order)
            .innerJoin(order.member, member)
            .where(
                limitedEditionOrder.audioContent.id.eq(contentId)
                    .and(limitedEditionOrder.order.member.id.eq(memberId))
            )
            .fetchFirst()
    }
}
