package kr.co.vividnext.sodalive.creator.admin.content.category

import com.querydsl.jpa.impl.JPAQueryFactory
import kr.co.vividnext.sodalive.content.QAudioContent.audioContent
import kr.co.vividnext.sodalive.content.category.QCategoryContent.categoryContent
import org.springframework.stereotype.Repository

@Repository
class CreatorAdminCategoryRepository(private val queryFactory: JPAQueryFactory) {
    fun searchContentNotInCategory(
        categoryId: Long,
        searchWord: String,
        memberId: Long
    ): List<SearchContentNotInCategoryResponse> {
        return queryFactory
            .select(QSearchContentNotInCategoryResponse(audioContent.id, audioContent.title))
            .from(audioContent)
            .leftJoin(categoryContent)
            .on(
                audioContent.id.eq(categoryContent.content.id)
                    .and(categoryContent.category.id.eq(categoryId))
            )
            .where(
                audioContent.duration.isNotNull
                    .and(audioContent.member.isNotNull)
                    .and(audioContent.member.id.eq(memberId))
                    .and(audioContent.isActive.isTrue.or(audioContent.releaseDate.isNotNull))
                    .and(audioContent.title.contains(searchWord))
                    .and(categoryContent.id.isNull.or(categoryContent.isActive.isFalse))
            )
            .fetch()
    }

    fun getContentInCategoryTotalCount(categoryId: Long, memberId: Long): Int {
        return queryFactory
            .select(audioContent.id)
            .from(audioContent)
            .leftJoin(categoryContent)
            .on(audioContent.id.eq(categoryContent.content.id).and(categoryContent.isActive.isTrue))
            .where(
                categoryContent.category.id.eq(categoryId)
                    .and(audioContent.duration.isNotNull)
                    .and(audioContent.member.isNotNull)
                    .and(audioContent.member.id.eq(memberId))
                    .and(audioContent.isActive.isTrue.or(audioContent.releaseDate.isNotNull))
            )
            .fetch()
            .size
    }

    fun getContentInCategory(
        imageHost: String,
        categoryId: Long,
        memberId: Long,
        offset: Long,
        limit: Long
    ): List<GetContentInCategoryItem> {
        return queryFactory
            .select(
                QGetContentInCategoryItem(
                    audioContent.id,
                    audioContent.title,
                    audioContent.isAdult,
                    audioContent.coverImage.prepend("/").prepend(imageHost)
                )
            )
            .from(audioContent)
            .leftJoin(categoryContent)
            .on(audioContent.id.eq(categoryContent.content.id).and(categoryContent.isActive.isTrue))
            .where(
                categoryContent.category.id.eq(categoryId)
                    .and(audioContent.duration.isNotNull)
                    .and(audioContent.member.isNotNull)
                    .and(audioContent.member.id.eq(memberId))
                    .and(audioContent.isActive.isTrue.or(audioContent.releaseDate.isNotNull))
            )
            .orderBy(categoryContent.updatedAt.desc())
            .offset(offset)
            .limit(limit)
            .fetch()
    }
}
