package kr.co.vividnext.sodalive.creator.admin.content.series.genre

import com.querydsl.jpa.impl.JPAQueryFactory
import kr.co.vividnext.sodalive.admin.content.series.genre.QSeriesGenre.seriesGenre
import kr.co.vividnext.sodalive.admin.content.series.genre.SeriesGenre
import org.springframework.data.jpa.repository.JpaRepository

interface CreatorAdminContentSeriesGenreRepository :
    JpaRepository<SeriesGenre, Long>, CreatorAdminContentSeriesGenreQueryRepository

interface CreatorAdminContentSeriesGenreQueryRepository {
    fun findActiveSeriesGenreById(id: Long): SeriesGenre
    fun getGenreList(isAdult: Boolean): List<GetGenreListResponse>
}

class CreatorAdminContentSeriesGenreQueryRepositoryImpl(
    private val queryFactory: JPAQueryFactory
) : CreatorAdminContentSeriesGenreQueryRepository {
    override fun findActiveSeriesGenreById(id: Long): SeriesGenre {
        return queryFactory
            .selectFrom(seriesGenre)
            .where(
                seriesGenre.id.eq(id)
                    .and(seriesGenre.isActive.isTrue)
            )
            .fetchFirst()
    }

    override fun getGenreList(isAdult: Boolean): List<GetGenreListResponse> {
        var where = seriesGenre.isActive.isTrue

        if (!isAdult) {
            where = where.and(seriesGenre.isAdult.isFalse)
        }

        return queryFactory
            .select(QGetGenreListResponse(seriesGenre.id, seriesGenre.genre))
            .from(seriesGenre)
            .where(where)
            .orderBy(seriesGenre.orders.asc())
            .fetch()
    }
}
