package kr.co.vividnext.sodalive.fcm

import kr.co.vividnext.sodalive.member.MemberRepository
import org.springframework.data.repository.findByIdOrNull
import org.springframework.stereotype.Service
import org.springframework.transaction.annotation.Transactional

@Service
class PushTokenService(
    private val repository: PushTokenRepository,
    private val memberRepository: MemberRepository
) {
    @Transactional
    fun registerToken(memberId: Long, token: String, deviceType: String) {
        val member = memberRepository.findByIdOrNull(memberId)

        if (member != null) {
            val existing = repository.findByToken(token)
            if (existing != null) {
                existing.member = member
                existing.token = token
                existing.deviceType = deviceType
            } else {
                val newToken = PushToken(token, deviceType)
                newToken.member = member
                repository.save(newToken)
            }
        }
    }

    @Transactional
    fun logout(memberId: Long) {
        val tokens = repository.findByMemberId(memberId)

        for (token in tokens) {
            token.member = null
        }
    }

    @Transactional
    fun unregisterInvalidToken(token: String) {
        val existing = repository.findByToken(token)
        if (existing != null) {
            repository.delete(existing)
        }
    }
}
